@@ -7,7 +7,7 @@ mod util;
 
 use event::State;
 use grid::{builder::Builder, Grid};
-use std::{borrow::Cow, io::Write, process, sync, time::Duration};
+use std::{borrow::Cow, fs, io, io::Write, process, sync, time::Duration};
 use terminal::{
     util::{Color, Point, Size},
     Terminal,
@@ -16,9 +16,12 @@ use terminal::{
 // Things that could be implemented but might not be worth it:
 // -A main menu
 // -An interactive tutorial
-// -Currently whole clue rows are grayed out once all cells for those clues have been solved.
+// -Currently whole clue rows are grayed out once all cells for those clues have been solved
 //  Make them gray out individually. (Maybe itertools' `pad_using` is helpful)
-// -Make the builder be able to build non-squared grids. Currently it doesn't work when width != height.
+// -Ability to change grid size and load grids (with F5?) within the game without the command line
+// -Ability to save records to a file and determine new records with that
+// -A ruler/measurer (with shift?)
+// -Ability to continue after solving the puzzle/ability to play it again
 
 fn main() {
     let code = match run() {
@@ -68,9 +71,9 @@ fn run() -> Result<(), Cow<'static, str>> {
     {
         let writer_arc = sync::Arc::new(sync::Mutex::new(writer));
 
-        let ctrlc_writer = writer_arc.clone();
+        let mut ctrlc_writer = writer_arc.clone();
         ctrlc::set_handler(move || {
-            ctrlc_writer.lock().unwrap().flush().unwrap();
+            write_grid_back(&mut ctrlc_writer);
         })
         .unwrap();
 
@@ -111,13 +114,18 @@ fn run() -> Result<(), Cow<'static, str>> {
         }
     }
 
-    if let Some(writer) = writer_arc {
-        writer.lock().unwrap().flush().unwrap();
+    if let Some(mut writer) = writer_arc {
+        write_grid_back(&mut writer);
     }
 
     Ok(())
 }
 
+fn write_grid_back(writer: &mut sync::Arc<sync::Mutex<io::BufWriter<fs::File>>>) {
+    // If flushing failed there is not much we can do.
+    writer.lock().unwrap().flush().ok();
+}
+
 fn draw_help(terminal: &mut Terminal, builder: &Builder) {
     terminal.set_foreground_color(Color::DarkGray);
     draw_text(
@@ -129,7 +137,7 @@ fn draw_help(terminal: &mut Terminal, builder: &Builder) {
     terminal.reset_colors();
 }
 
-fn get_grid(arg: &Result<Option<args::Arg>, &'static str>) -> Result<Grid, Cow<'static, str>> {
+fn get_grid(arg: &Result<Option<args::Arg>, Cow<'static, str>>) -> Result<Grid, Cow<'static, str>> {
     match arg {
         Ok(arg) => match arg {
             Some(args::Arg::File {
@@ -159,7 +167,7 @@ fn get_grid(arg: &Result<Option<args::Arg>, &'static str>) -> Result<Grid, Cow<'
                 Ok(Grid::random(grid_size))
             }
         },
-        Err(err) => Err((*err).into()),
+        Err(err) => Err(err.clone()),
     }
 }
 
